﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using BeatLyrics.Tool.Models;

namespace BeatLyrics.Tool.Dialogs
{
    public partial class SplitForm : Form
    {
        private readonly List<LyricDetailExt> _items;
        private readonly LyricDetailExt _hoverObj;
        private readonly int _playPosMs;

        public SplitForm(List<LyricDetailExt> items, LyricDetailExt hoverObj, int playPosMs)
        {
            _items = items;
            _hoverObj = hoverObj;
            _playPosMs = playPosMs;
            InitializeComponent();
        }

        private void UpdateSplit()
        {
            TopTextBox.Text = _hoverObj.Text.Substring(0, SplitTrackBar.Value).Trim();
            BottomTextBox.Text = _hoverObj.Text.Substring(SplitTrackBar.Value).Trim();
        }

        private void SplitForm_Shown(object sender, EventArgs e)
        {
            var topOffset = Top - PointToScreen(Point.Empty).Y;


            Left = MousePosition.X - DoneButton.Left - DoneButton.Width / 2;
            Top = MousePosition.Y - DoneButton.Top - DoneButton.Height / 2 + topOffset;
            SplitTrackBar.Maximum = _hoverObj.Text.Length - 1;

            var splitPos = _hoverObj.Text.IndexOf(' ');
            if (splitPos == -1) splitPos = _hoverObj.Text.IndexOf('　');
            if (splitPos == -1) splitPos = SplitTrackBar.Maximum / 2;
            SplitTrackBar.Value = splitPos;

            UpdateSplit();
        }

        private void SplitTrackBar_Scroll(object sender, EventArgs e)
        {
            UpdateSplit();
        }

        private void DoneButton_Click(object sender, EventArgs e)
        {
            var index = _items.IndexOf(_hoverObj);

            var add = new LyricDetailExt
            {
                Text = BottomTextBox.Text,
                TimeMs = _playPosMs,
                DurationMs = _hoverObj.TimeMs + _hoverObj.DurationMs - _playPosMs
            };
            _items.Insert(index + 1, add);

            _hoverObj.Text = TopTextBox.Text;
            _hoverObj.DurationMs = _playPosMs - _hoverObj.TimeMs;

            Close();
        }
    }
}
