﻿using BeatLyrics.Common;
using BeatLyrics.Common.Models;
using HMUI;
using Newtonsoft.Json;
using System;
using System.Collections;
using System.IO;
using System.Linq;
using TMPro;
using UnityEngine;

namespace BeatLyrics
{
    public class LyricsComponent : MonoBehaviour
    {
        private static readonly Quaternion Rot = Quaternion.Euler(new Vector3(0, 0, 0));
        private static readonly Vector3 Pos = new Vector3(0, 1.35f, 15f);
        private static readonly float FontSize = 50;
        private static readonly Color32 FontColor = new Color32(255, 255, 255, 96);
        private static readonly float OutlineWidth = 0.4f;
        private static readonly Color32 OutlineColor = new Color32(255, 0, 0, 255);

        //TODO: OutLine
        //TODO: Select Font

        private readonly AudioTimeSyncController _audioTime;

        private readonly GameObject _textCanvas = null;
        private readonly RubyTextMeshProUGUI _text = null;

        private GameplayCoreSceneSetupData _data;
        private LyricFile _lyricFile;
        private string _lyricFileName;

        private string _debugInfo;

        public LyricsComponent()
        {
            _audioTime = Resources.FindObjectsOfTypeAll<AudioTimeSyncController>().FirstOrDefault();

            _textCanvas = new GameObject();
            _textCanvas.AddComponent<Canvas>();

            _textCanvas.name = "BeatLyrics Canvas";
            _textCanvas.transform.position = Pos;
            _textCanvas.transform.rotation = Rot;
            _textCanvas.transform.localScale = new Vector3(0.01f, 0.01f, 1.0f);

            var textObj = new GameObject();
            textObj.transform.SetParent(_textCanvas.transform);
            textObj.transform.localPosition = Vector3.zero;
            textObj.transform.localRotation = Quaternion.identity;
            textObj.transform.localScale = Vector3.one;

            _text = textObj.AddComponent<RubyTextMeshProUGUI>();

            _textCanvas.AddComponent<CurvedCanvasSettings>().SetRadius(0f);

            _text.alignment = TextAlignmentOptions.Center;
            _text.fontSize = FontSize;
            _text.enableWordWrapping = false;
            _text.color = FontColor;
            _text.name = "Lyrics Text";
            _text.text = "";

            //text.font=
        }

        public IEnumerator Start()
        {
            _data = BS_Utils.Plugin.LevelData.GameplayCoreSceneSetupData;

            Plugin.Logger.Debug("GameplayCoreSceneSetupData is null:" + (_data == null));

            try
            {
                LoadLrc();
            }
            catch (Exception e)
            {
                Plugin.Logger.Error(e);
                _debugInfo = "Error Load Lrc:" + e.Message;
            }

            yield break;
        }

        private string LoadLrc()
        {
            if (_data?.difficultyBeatmap.level.levelID != null)
            {
                var beatLyricsDir = DirProvider.BeatLyricsDir;

                if (beatLyricsDir != null)
                {
                    var path = GetSetFilePath(_data.difficultyBeatmap.level.levelID);
                    if (File.Exists(path))
                    {
                        var jsonFileName = File.ReadAllText(path);
                        var jsonFilePath = Path.Combine(beatLyricsDir, _data.difficultyBeatmap.level.levelID, jsonFileName);
                        if (File.Exists(jsonFilePath))
                        {
                            var json = File.ReadAllText(jsonFilePath);
                            _lyricFile = JsonConvert.DeserializeObject<LyricFile>(json);
                            _lyricFileName = Path.GetFileNameWithoutExtension(jsonFileName);
                            Plugin.Logger.Debug("json file loaded: " + jsonFilePath);
                            return _lyricFileName;
                        }
                        else
                        {
                            Plugin.Logger.Debug("json file not found");
                            _debugInfo += "json file not found";
                            return "json file not found";
                        }
                    }
                    else
                    {
                        Plugin.Logger.Debug("set.txt file not found");
                        _debugInfo += "set.txt file not found";
                        return "set.txt file not found";
                    }
                }
                else
                {
                    Plugin.Logger.Debug("failure to get EnvVar");
                    _debugInfo += "failure to get EnvVar";
                    return "failure to get EnvVar";
                }
            }

            return "Unknown";
        }

        public void Update()
        {
            var milliseconds = _audioTime?.songTime * 1000 ?? 0;

            if (milliseconds > 0)
            {
                if (_lyricFile != null)
                {
                    var lrc = _lyricFile.Main?.Where(p => p.TimeMs <= milliseconds && p.TimeMs + p.DurationMs >= milliseconds).Select(p => p.Text).FirstOrDefault();
                    var sub = _lyricFile.Subtitle?.FirstOrDefault(p => p.TimeMs <= milliseconds && p.TimeMs + p.DurationMs >= milliseconds)?.Text;

                    if (milliseconds < 5000 && lrc == null && sub == null)
                    {
                        _text.text = _lyricFileName;
                    }
                    else
                    {
                        //Show LRC
                        _text.UnditedText = lrc + Environment.NewLine + sub;
                    }
                }
                else if (milliseconds < 3000)
                {
                    var level = _data?.difficultyBeatmap.level;
                    _text.text = "此处应有歌词,但是未能加载" +
                                 $"{Environment.NewLine}{level?.levelID}" +
                                 $"{Environment.NewLine}{_debugInfo}";
                }
                else
                {
                    _text.text = "";
                }
            }
            else
            {
                _text.text = "";
            }
        }

        public static string GetSetFilePath(string levelId)
        {
            return Path.Combine(DirProvider.BeatLyricsDir, levelId + DirProvider.SuffixSet);
        }
    }
}
