﻿using IPA;
using UnityEngine.SceneManagement;
using IPALogger = IPA.Logging.Logger;

namespace BeatLyrics
{
    /// <summary>
    ///   Entry point of the plugin.
    /// </summary>
    [Plugin(RuntimeOptions.SingleStartInit)]
    public sealed class Plugin //: IPlugin
    {
        internal static IPALogger Logger;

        [Init]
        public void Init(IPALogger logger)
        {
            Logger = logger;
            Logger.Debug("Init");
        }

        private LyricsComponent _lastUsedLyricsComponent;

        [OnStart]
        public void OnApplicationStart()
        {
            Logger.Debug("OnApplicationStart");
            SceneManager.activeSceneChanged += OnActiveSceneChanged;
        }

        [OnExit]
        public void OnApplicationQuit()
        {
            Logger.Debug("OnApplicationQuit");
            SceneManager.activeSceneChanged -= OnActiveSceneChanged;
        }

        #region Unused

        public void OnLevelWasInitialized(int level)
        {
        }

        public void OnLevelWasLoaded(int level)
        {
        }

        public void OnUpdate()
        {
        }

        public void OnFixedUpdate()
        {
        }

        #endregion Unused

        public void OnActiveSceneChanged(Scene _, Scene newScene)
        {
            Logger.Debug("OnActiveSceneChanged: " + _.name + " to " + newScene.name);
            if (newScene.name == "GameCore")
            {
                var rootGameObjects = newScene.GetRootGameObjects();
                Logger.Debug("GetRootGameObjects -> " + rootGameObjects.Length);

                Logger.Debug("Create LyricsComponent");
                _lastUsedLyricsComponent = rootGameObjects[0].AddComponent<LyricsComponent>();
                Logger.Debug("Create LyricsComponent OK..?");
            }
            else
            {
                if (_lastUsedLyricsComponent != null)
                {
                    //TODO: free resources
                }
            }
        }
    }
}
